#ifndef _MONO_BURG_DEFS_
#define _MONO_BURG_DEFS_

/*
 * x86.brg: X86 code generator
 *
 * Author:
 *   Dietmar Maurer (dietmar@ximian.com)
 *
 * (C) 2001 Ximian, Inc.
 */

#include <config.h>

#include <glib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#ifndef PLATFORM_WIN32
#include <signal.h>
#include <sys/syscall.h>
#endif

#include <mono/metadata/blob.h>
#include <mono/metadata/metadata.h>
#include <mono/metadata/loader.h>
#include <mono/metadata/object.h>
#include <mono/metadata/tabledefs.h>
#include <mono/metadata/appdomain.h>
#include <mono/metadata/marshal.h>
#include <mono/arch/x86/x86-codegen.h>

#include "regset.h"
#include "jit.h"

/*
 * Pull the list of opcodes
 */
#define OPDEF(a,b,c,d,e,f,g,h,i,j) \
	a = i,

enum {
#include "mono/cil/opcode.def"
	LAST = 0xff
};
#undef OPDEF

/* alignment of activation frames */
#define MONO_FRAME_ALIGNMENT 4

void print_lmf (void);

#define MBTREE_TYPE  MBTree
#define MBCGEN_TYPE  MonoFlowGraph
#define MBCOST_DATA  MonoFlowGraph
#define MBALLOC_STATE mono_mempool_alloc (data->mp, sizeof (MBState))

typedef enum {
	AMImmediate       = 0,  // ptr
	AMBase            = 1,  // V[REG]  
	AMIndex           = 2,  // V[REG*X] 
	AMBaseIndex       = 3,  // V[REG*X][REG] 
} X86AddMode;

typedef	struct {
	int           offset;
	X86AddMode    amode:2;
	unsigned int  shift:2;
	gint8         basereg;
	gint8         indexreg;
} X86AddressInfo;

struct _MBTree {
	guint16   op;
	unsigned  last_instr:1;
	unsigned  spilled:1;
	
	MBTree   *left, *right;
	gpointer  state;
	gpointer  emit;

	gint32    addr;
	gint32    cli_addr;

	gint8     reg1;
	gint8     reg2;
	gint8     reg3;
	
	MonoValueType svt;

	union {
		gint32 i;
		gint64 l;
		gpointer p;
		MonoMethod *m;
		MonoBBlock *bb;
		MonoClass *klass;
		MonoClassField *field;
		X86AddressInfo ainfo;
		MonoJitFieldInfo fi;
		MonoJitBranchInfo bi;
		MonoJitCallInfo call_info;	  
		MonoJitArgumentInfo arg_info;	  
	} data;
};

gint64   mono_llmult        (gint64 a, gint64 b);
guint64  mono_llmult_ovf    (gpointer *exc, guint32 al, gint32 ah, guint32 bl, gint32 bh);
guint64  mono_llmult_ovf_un (gpointer *exc, guint32 al, guint32 ah, guint32 bl, guint32 bh);
gint64   mono_lldiv         (gint64 a, gint64 b);
gint64   mono_llrem         (gint64 a, gint64 b);
guint64  mono_lldiv_un      (guint64 a, guint64 b);
guint64  mono_llrem_un      (guint64 a, guint64 b);
gpointer mono_ldsflda       (MonoClass *klass, int offset);

gpointer mono_ldvirtftn     (MonoObject *this, int slot);
gpointer mono_ldintftn      (MonoObject *this, int slot);
gpointer mono_ldftn         (MonoMethod *method);

void mono_emit_fast_iconv(MBCGEN_TYPE* s, MBTREE_TYPE* tree);
void mono_emit_fast_iconv_i8(MBCGEN_TYPE* s, MBTREE_TYPE* tree);

MonoArray*
mono_array_new_wrapper  (MonoClass *eclass, guint32 n);
MonoObject *
mono_object_new_wrapper (MonoClass *klass);
MonoString*
mono_ldstr_wrapper      (MonoImage *image, guint32 ind);

gpointer
get_mono_object_isinst (void);

#define MB_OPT_LEVEL 1

#if MB_OPT_LEVEL == 0
#define MB_USE_OPT1(c) 65535
#define MB_USE_OPT2(c) 65535
#endif
#if MB_OPT_LEVEL == 1
#define MB_USE_OPT1(c) c
#define MB_USE_OPT2(c) 65535
#endif
#if MB_OPT_LEVEL >= 2
#define MB_USE_OPT1(c) c
#define MB_USE_OPT2(c) c
#endif

//#define DEBUG

#define REAL_PRINT_REG(text,reg) \
mono_assert (reg >= 0); \
x86_push_reg (s->code, X86_EAX); \
x86_push_reg (s->code, X86_EDX); \
x86_push_reg (s->code, X86_ECX); \
x86_push_reg (s->code, reg); \
x86_push_imm (s->code, reg); \
x86_push_imm (s->code, text " %d %p\n"); \
x86_mov_reg_imm (s->code, X86_EAX, printf); \
x86_call_reg (s->code, X86_EAX); \
x86_alu_reg_imm (s->code, X86_ADD, X86_ESP, 3*4); \
x86_pop_reg (s->code, X86_ECX); \
x86_pop_reg (s->code, X86_EDX); \
x86_pop_reg (s->code, X86_EAX);

void *
debug_memcopy (void *dest, const void *src, size_t n);

#ifdef DEBUG
#define MEMCOPY debug_memcopy
#define PRINT_REG(text,reg) REAL_PRINT_REG(text,reg)
#else

#define MEMCOPY memcpy

#define PRINT_REG(x,y)

#endif

/* The call instruction for virtual functions must have a known
 * size (used by x86_magic_trampoline)
 */
#define x86_call_virtual(inst,basereg,disp)	               \
	do {	                                               \
		*(inst)++ = (unsigned char)0xff;               \
		x86_address_byte ((inst), 2, 2, (basereg));    \
		x86_imm_emit32 ((inst), (disp));	       \
	} while (0)

/* emit an exception if condition is fail */
#define EMIT_COND_SYSTEM_EXCEPTION(cond,signed,exc_name)       \
        do {                                                   \
                gpointer t;                                    \
	        x86_branch8 (s->code, cond, 10, signed);       \
	        x86_push_imm (s->code, exc_name);              \
		t = arch_get_throw_exception_by_name ();       \
		mono_add_jump_info (s, s->code,                \
                                    MONO_JUMP_INFO_ABS, t);    \
	        x86_call_code (s->code, 0);                    \
	} while (0); 

#define X86_ARG_PAD(pad) do {                                               \
	if (pad) {                                                          \
		if (pad == 4)                                               \
			x86_push_reg (s->code, X86_EAX);                    \
		else                                                        \
			x86_alu_reg_imm (s->code, X86_SUB, X86_ESP, pad);   \
	}                                                                   \
} while (0)

#define X86_CALL_END    do {                                       \
        int size = tree->data.call_info.frame_size;                \
        if (size)                                                  \
	        x86_alu_reg_imm (s->code, X86_ADD, X86_ESP, size); \
} while (0)

#define X86_CALL_BEGIN  do {                                                     \
        int pad = tree->data.call_info.pad;                                      \
        X86_ARG_PAD (pad);                                                       \
	if (tree->left->op != MB_TERM_NOP) {                                     \
		mono_assert (lreg >= 0);                                         \
		x86_push_reg (s->code, lreg);                                    \
		x86_alu_membase_imm (s->code, X86_CMP, lreg, 0, 0);              \
	}                                                                        \
	if (tree->data.call_info.vtype_num) {                                    \
		int offset = VARINFO (s, tree->data.call_info.vtype_num).offset; \
		x86_lea_membase (s->code, treg, X86_EBP, offset);                \
		x86_push_reg (s->code, treg);                                    \
	}                                                                        \
} while (0)

/* we use this macro to move one lreg to another - source and
   destination may overlap, but the register allocator has to
   make sure that ((d1 < d2) && (s1 < s2))
*/
#define MOVE_LREG(d1,d2,s1,s2)                                        \
        do {                                                          \
	        g_assert ((d1 < d2) && (s1 < s2));                    \
	        if ((d1) <= (s1)) {                                   \
			if ((d1) != (s1))                             \
				x86_mov_reg_reg (s->code, d1, s1, 4); \
			if ((d2) != (s2))                             \
				x86_mov_reg_reg (s->code, d2, s2, 4); \
		} else {                                              \
			if ((d2) != (s2))                             \
				x86_mov_reg_reg (s->code, d2, s2, 4); \
			if ((d1) != (s1))                             \
				x86_mov_reg_reg (s->code, d1, s1, 4); \
		}                                                     \
	} while (0); 
        

#include <glib.h>

#ifndef MBTREE_TYPE
#error MBTREE_TYPE undefined
#endif
#ifndef MBTREE_OP
#define MBTREE_OP(t) ((t)->op)
#endif
#ifndef MBTREE_LEFT
#define MBTREE_LEFT(t) ((t)->left)
#endif
#ifndef MBTREE_RIGHT
#define MBTREE_RIGHT(t) ((t)->right)
#endif
#ifndef MBTREE_STATE
#define MBTREE_STATE(t) ((t)->state)
#endif
#ifndef MBCGEN_TYPE
#define MBCGEN_TYPE int
#endif
#ifndef MBALLOC_STATE
#define MBALLOC_STATE g_new (MBState, 1)
#endif
#ifndef MBCOST_DATA
#define MBCOST_DATA gpointer
#endif

#define MBMAXCOST 32768

#define MBCOND(x) if (!(x)) return MBMAXCOST;

#define MB_TERM_CONST_I4	 1
#define MB_TERM_CONST_I8	 2
#define MB_TERM_CONST_R4	 3
#define MB_TERM_CONST_R8	 4
#define MB_TERM_LDIND_I1	 5
#define MB_TERM_LDIND_U1	 6
#define MB_TERM_LDIND_I2	 7
#define MB_TERM_LDIND_U2	 8
#define MB_TERM_LDIND_I4	 9
#define MB_TERM_LDIND_I8	 10
#define MB_TERM_LDIND_R4	 11
#define MB_TERM_LDIND_R8	 12
#define MB_TERM_LDIND_OBJ	 13
#define MB_TERM_STIND_I1	 14
#define MB_TERM_STIND_I2	 15
#define MB_TERM_STIND_I4	 16
#define MB_TERM_STIND_I8	 17
#define MB_TERM_STIND_R4	 18
#define MB_TERM_STIND_R8	 19
#define MB_TERM_STIND_OBJ	 20
#define MB_TERM_ADDR_L	 21
#define MB_TERM_ADDR_G	 22
#define MB_TERM_ARG_I4	 23
#define MB_TERM_ARG_I8	 24
#define MB_TERM_ARG_R4	 25
#define MB_TERM_ARG_R8	 26
#define MB_TERM_ARG_OBJ	 27
#define MB_TERM_CALL_I4	 28
#define MB_TERM_CALL_I8	 29
#define MB_TERM_CALL_R8	 30
#define MB_TERM_CALL_VOID	 31
#define MB_TERM_BREAK	 32
#define MB_TERM_SWITCH	 33
#define MB_TERM_BR	 34
#define MB_TERM_RET_VOID	 35
#define MB_TERM_RET	 36
#define MB_TERM_RET_OBJ	 37
#define MB_TERM_ENDFINALLY	 38
#define MB_TERM_ENDFILTER	 39
#define MB_TERM_JMP	 40
#define MB_TERM_ADD	 41
#define MB_TERM_ADD_OVF	 42
#define MB_TERM_ADD_OVF_UN	 43
#define MB_TERM_SUB	 44
#define MB_TERM_SUB_OVF	 45
#define MB_TERM_SUB_OVF_UN	 46
#define MB_TERM_MUL	 47
#define MB_TERM_MUL_OVF	 48
#define MB_TERM_MUL_OVF_UN	 49
#define MB_TERM_DIV	 50
#define MB_TERM_DIV_UN	 51
#define MB_TERM_REM	 52
#define MB_TERM_REM_UN	 53
#define MB_TERM_AND	 54
#define MB_TERM_OR	 55
#define MB_TERM_XOR	 56
#define MB_TERM_SHL	 57
#define MB_TERM_SHR	 58
#define MB_TERM_SHR_UN	 59
#define MB_TERM_NEG	 60
#define MB_TERM_NOT	 61
#define MB_TERM_CKFINITE	 62
#define MB_TERM_COMPARE	 63
#define MB_TERM_CBRANCH	 64
#define MB_TERM_BRTRUE	 65
#define MB_TERM_BRFALSE	 66
#define MB_TERM_CSET	 67
#define MB_TERM_CONV_I4	 68
#define MB_TERM_CONV_I1	 69
#define MB_TERM_CONV_I2	 70
#define MB_TERM_CONV_I8	 71
#define MB_TERM_CONV_U1	 72
#define MB_TERM_CONV_U2	 73
#define MB_TERM_CONV_U4	 74
#define MB_TERM_CONV_U8	 75
#define MB_TERM_CONV_R4	 76
#define MB_TERM_CONV_R8	 77
#define MB_TERM_CONV_R_UN	 78
#define MB_TERM_INTF_ADDR	 79
#define MB_TERM_VFUNC_ADDR	 80
#define MB_TERM_NOP	 81
#define MB_TERM_NEWARR	 82
#define MB_TERM_NEWARR_SPEC	 83
#define MB_TERM_NEWOBJ	 84
#define MB_TERM_NEWOBJ_SPEC	 85
#define MB_TERM_INITBLK	 86
#define MB_TERM_CPBLK	 87
#define MB_TERM_CPSRC	 88
#define MB_TERM_POP	 89
#define MB_TERM_INITOBJ	 90
#define MB_TERM_LOCALLOC	 91
#define MB_TERM_ISINST	 92
#define MB_TERM_CASTCLASS	 93
#define MB_TERM_UNBOX	 94
#define MB_TERM_CONV_OVF_I1	 95
#define MB_TERM_CONV_OVF_U1	 96
#define MB_TERM_CONV_OVF_I2	 97
#define MB_TERM_CONV_OVF_U2	 98
#define MB_TERM_CONV_OVF_U4	 99
#define MB_TERM_CONV_OVF_U8	 100
#define MB_TERM_CONV_OVF_I4	 101
#define MB_TERM_CONV_OVF_I4_UN	 102
#define MB_TERM_CONV_OVF_U1_UN	 103
#define MB_TERM_CONV_OVF_U2_UN	 104
#define MB_TERM_CONV_OVF_I2_UN	 105
#define MB_TERM_CONV_OVF_I8_UN	 106
#define MB_TERM_CONV_OVF_I1_UN	 107
#define MB_TERM_EXCEPTION	 108
#define MB_TERM_THROW	 109
#define MB_TERM_RETHROW	 110
#define MB_TERM_HANDLER	 111
#define MB_TERM_CHECKTHIS	 112
#define MB_TERM_LDLEN	 113
#define MB_TERM_LDELEMA	 114
#define MB_TERM_LDFTN	 115
#define MB_TERM_LDVIRTFTN	 116
#define MB_TERM_LDSTR	 117
#define MB_TERM_LDSFLDA	 118
#define MB_TERM_REMOTE_LDFLDA	 119
#define MB_TERM_REMOTE_STIND_I1	 120
#define MB_TERM_REMOTE_STIND_I2	 121
#define MB_TERM_REMOTE_STIND_I4	 122
#define MB_TERM_REMOTE_STIND_I8	 123
#define MB_TERM_REMOTE_STIND_R4	 124
#define MB_TERM_REMOTE_STIND_R8	 125
#define MB_TERM_REMOTE_STIND_OBJ	 126
#define MB_TERM_SIN	 127
#define MB_TERM_COS	 128
#define MB_TERM_SQRT	 129
#define MB_TERM_FUNC1	 130
#define MB_TERM_PROC2	 131
#define MB_TERM_PROC3	 132
#define MB_TERM_FREE	 133
#define MB_TERM_OBJADDR	 134
#define MB_TERM_VTADDR	 135

#define MB_NTERM_stmt	1
#define MB_NTERM_acon	2
#define MB_NTERM_base	3
#define MB_NTERM_reg	4
#define MB_NTERM_index	5
#define MB_NTERM_addr	6
#define MB_NTERM_cflags	7
#define MB_NTERM_this	8
#define MB_NTERM_lreg	9
#define MB_NTERM_freg	10
#define MB_MAX_NTERMS	10

typedef struct _MBState MBState;
struct _MBState {
	int		 op;
	MBState		*left, *right;
	guint16		cost[11];
	unsigned int	 rule_stmt:7;
	unsigned int	 rule_acon:2;
	unsigned int	 rule_base:3;
	unsigned int	 rule_reg:7;
	unsigned int	 rule_index:2;
	unsigned int	 rule_addr:2;
	unsigned int	 rule_cflags:3;
	unsigned int	 rule_this:2;
	unsigned int	 rule_lreg:6;
	unsigned int	 rule_freg:5;
};

typedef void (*MBEmitFunc) (MBTREE_TYPE *tree, MBCGEN_TYPE *s);

extern const char * const mono_burg_term_string [];
extern const char * const mono_burg_rule_string [];
extern const guint16 *const mono_burg_nts [];
extern MBEmitFunc const mono_burg_func [];
MBState *mono_burg_label (MBTREE_TYPE *tree, MBCOST_DATA *data);
int mono_burg_rule (MBState *state, int goal);
MBTREE_TYPE **mono_burg_kids (MBTREE_TYPE *tree, int rulenr, MBTREE_TYPE *kids []);

#endif /* _MONO_BURG_DEFS_ */
